#!/usr/bin/env node
var minimist = require('minimist');
var extend = require('xtend');
var typings_1 = require('../typings');
var cli_1 = require('../utils/cli');
var config_1 = require('../utils/config');
var registry_1 = require('../lib/registry');
var cli_2 = require('../utils/cli');
var args = minimist(process.argv.slice(2), {
    boolean: ['save', 'saveDev', 'ambient', 'verbose', 'help', 'production'],
    string: ['name', 'source'],
    alias: {
        save: ['S'],
        saveDev: ['save-dev', 'D'],
        name: ['n'],
        ambient: ['A'],
        verbose: ['v'],
        help: ['h'],
        source: ['s'],
        production: ['p']
    }
});
if (args.help) {
    console.log("\n" + config_1.PROJECT_NAME + " install (with no arguments, in package directory)\n" + config_1.PROJECT_NAME + " install <pkg>[@<version>] --source [" + registry_1.VALID_SOURCES.join(' | ') + "]\n" + config_1.PROJECT_NAME + " install file:<path>\n" + config_1.PROJECT_NAME + " install github:<github username>/<github project>[/<path>][#<commit>]\n" + config_1.PROJECT_NAME + " install bitbucket:<bitbucket username>/<bitbucket project>[/<path>][#<commit>]\n" + config_1.PROJECT_NAME + " install <http:// url>\n\nAliases: i, in\nOptions: [--name] [--save|--save-dev] [--ambient] [--production]\n");
    process.exit(0);
}
function installer(args) {
    var options = extend(args, { cwd: process.cwd() });
    if (!args._.length) {
        return cli_1.loader(typings_1.install(options), args)
            .then(function (tree) {
            console.log(cli_2.archifyDependencyTree(tree, { ambient: true, dev: true }));
        });
    }
    var dependency = args._[0];
    if (!registry_1.isRegistryPath(dependency)) {
        return cli_1.loader(typings_1.installDependency(dependency, options), args)
            .then(function (tree) {
            console.log(cli_2.archifyDependencyTree(tree, { name: options.name }));
        });
    }
    var _a = registry_1.parseRegistryPath(dependency), name = _a.name, version = _a.version;
    function installFrom(source) {
        var installationName = args.name || name;
        return registry_1.getVersions(source, name, version)
            .then(function (project) {
            var versions = project.versions;
            if (versions.length === 1) {
                return versions[0];
            }
            return cli_1.inquire([{
                    name: 'version',
                    type: 'list',
                    message: 'Select a version',
                    choices: versions.map(function (x, i) { return ({ name: x.version, value: String(i) }); })
                }])
                .then(function (answers) { return versions[answers.version]; });
        })
            .then(function (version) {
            var installation = typings_1.installDependency(version.location, extend(options, { name: installationName }));
            console.log("Installing " + name + "@" + version.version + " from " + source + "...");
            return cli_1.loader(installation, args);
        })
            .then(function (tree) {
            console.log(cli_2.archifyDependencyTree(tree, { name: installationName }));
        });
    }
    if (args.source) {
        return installFrom(args.source);
    }
    return cli_1.loader(registry_1.search({ name: name }), args)
        .then(function (result) {
        var results = result.results;
        if (results.length === 0) {
            return Promise.reject(new Error("Unable to find \"" + name + "\" in the registry"));
        }
        if (results.length === 1) {
            var item = results[0];
            return cli_1.inquire([{
                    type: 'confirm',
                    name: 'ok',
                    message: "Found \"" + name + "\" in the registry from \"" + item.source + "\". Ok?"
                }])
                .then(function (answers) {
                if (answers.ok) {
                    return installFrom(item.source);
                }
            });
        }
        return cli_1.inquire([{
                type: 'list',
                name: 'source',
                message: "Found \"" + name + "\" in the registry from multiple sources",
                choices: results.map(function (x) { return x.source; })
            }])
            .then(function (answers) { return installFrom(answers.source); });
    });
}
installer(args);
//# sourceMappingURL=typings-install.js.map